@@ -34,6 +34,7 @@
 #include "fr-command.h"
 #include "fr-command-arj.h"
 #include "fr-command-cfile.h"
+#include "fr-command-iso.h"
 #include "fr-command-lha.h"
 #include "fr-command-rar.h"
 #include "fr-command-rpm.h"
@@ -272,6 +273,9 @@ create_command_from_mime_type (FRArchive  *archive,
 	} else if (is_mime_type (mime_type, "application/x-rpm")) {
 		archive->command = fr_command_rpm_new (archive->process,
 						       filename);
+	} else if (is_mime_type (mime_type, "application/x-cd-image")) {
+		archive->command = fr_command_iso_new (archive->process,
+						       filename);
 	} else 
 		return FALSE;
 
@@ -428,6 +432,9 @@ create_command_from_filename (FRArchive  *archive,
 		} else if (file_extension_is (filename, ".rpm")) {
 			archive->command = fr_command_rpm_new (archive->process,
 							       filename);
+		} else if (file_extension_is (filename, ".iso")) {
+			archive->command = fr_command_iso_new (archive->process,
+							       filename);
 		} else
 			return FALSE;
 	} else
@@ -755,8 +762,6 @@ find_file_in_archive (FRArchive *archive,
 {
 	GList *scan;
 
-	g_return_val_if_fail (path != NULL, NULL);
-
 	for (scan = archive->command->file_list; scan; scan = scan->next) {
 		FileData *fdata = scan->data;
 		if (strcmp (path, fdata->original_path) == 0)
@@ -1389,6 +1394,84 @@ extract_in_chunks (FRCommand  *command,
 }
 
 
+static char*
+compute_base_path (const char *e_base_dir,
+		   const char *path)
+{
+	int         e_base_dir_len = strlen (e_base_dir);
+	int         path_len = strlen (path);
+	const char *base_path;
+	char       *name_end;
+	char       *new_path;
+
+	if (path_len <= e_base_dir_len)
+		return NULL;
+	base_path = path + e_base_dir_len;
+	if (path[0] != '/')
+		base_path -= 1;
+	name_end = strchr (base_path, '/');
+
+	if (name_end == NULL)
+		new_path = g_strdup (path);
+	else {
+		int name_len = name_end - path;
+		new_path = g_strndup (path, name_len);
+	}
+
+#ifdef DEBUG
+	g_print ("%s, %s --> %s\n", e_base_dir, path, new_path);
+#endif
+
+	return new_path;
+}
+
+
+static GList*
+compute_list_base_path (const char *base_dir,
+			GList      *e_filtered)
+{
+	char  *e_base_dir = shell_escape (base_dir);
+	GList *scan;
+	GList *list = NULL, *list_unique = NULL;
+	GList *last_inserted;
+
+	if (e_filtered == NULL)
+		return NULL;
+
+	for (scan = e_filtered; scan; scan = scan->next) {
+		const char *path = scan->data;
+		char       *new_path;
+		new_path = compute_base_path (e_base_dir, path);
+		if (new_path != NULL)
+			list = g_list_prepend (list, new_path);
+	}
+
+	/* The above operation can create duplicates, we remove them here. */
+	list = g_list_sort (list, (GCompareFunc)strcmp);
+	
+	last_inserted = NULL;
+	for (scan = list; scan; scan = scan->next) {
+		const char *path = scan->data;
+
+		if (last_inserted != NULL) {
+			const char *last_path = (const char*)last_inserted->data;
+			if (strcmp (last_path, path) == 0) {
+				g_free (scan->data);
+				continue;
+			}
+		}
+
+		last_inserted = scan;
+		list_unique = g_list_prepend (list_unique, scan->data);
+	}
+
+	g_list_free (list);
+	g_free (e_base_dir);
+
+	return list_unique;
+}
+
+
 /* Note : All paths unescaped.  
  * Note2: Do not escape dest_dir it will escaped in fr_command_extract if 
  *        needed. */
@@ -1396,6 +1479,7 @@ void
 fr_archive_extract (FRArchive  *archive,
 		    GList      *file_list,
 		    const char *dest_dir,
+		    const char *base_dir,
 		    gboolean    skip_older,
 		    gboolean    overwrite,
 		    gboolean    junk_paths,
@@ -1404,6 +1488,7 @@ fr_archive_extract (FRArchive  *archive,
 	GList    *filtered, *e_filtered;
 	GList    *scan;
 	gboolean  extract_all;
+	gboolean  use_base_dir;
 	gboolean  move_to_dest_dir;
 
 	g_return_if_fail (archive != NULL);
@@ -1413,7 +1498,12 @@ fr_archive_extract (FRArchive  *archive,
 	/* if a command supports all the requested options use 
 	 * fr_command_extract directly. */
 
-	if (! (! overwrite && ! archive->command->propExtractCanAvoidOverwrite)
+	use_base_dir = ! ((base_dir == NULL) 
+			  || (strcmp (base_dir, "") == 0)
+			  || (strcmp (base_dir, "/") == 0));
+		
+	if (! use_base_dir
+	    && ! (! overwrite && ! archive->command->propExtractCanAvoidOverwrite)
 	    && ! (skip_older && ! archive->command->propExtractCanSkipOlder)
 	    && ! (junk_paths && ! archive->command->propExtractCanJunkPaths)) {
 		GList *e_file_list;
@@ -1432,9 +1522,10 @@ fr_archive_extract (FRArchive  *archive,
 	}
 
 	/* .. else we have to implement the unsupported options. */
-
-	move_to_dest_dir = (junk_paths 
-			    && ! archive->command->propExtractCanJunkPaths);
+	
+	move_to_dest_dir = (use_base_dir
+			    || ((junk_paths 
+				 && ! archive->command->propExtractCanJunkPaths)));
 
 	extract_all = (file_list == NULL);
 	if (extract_all) {
@@ -1520,6 +1611,13 @@ fr_archive_extract (FRArchive  *archive,
 				   skip_older,
 				   junk_paths,
 				   password);
+
+		if (use_base_dir) {
+			GList *tmp = compute_list_base_path (base_dir, e_filtered);
+			path_list_free (e_filtered);
+			e_filtered = tmp;
+		}
+
 		move_files_in_chunks (archive, 
 				      e_filtered, 
 				      temp_dir, 
@@ -1582,6 +1680,7 @@ fr_archive_utils__get_file_name_ext (const char *filename)
 		".bz2", 
 		".ear",
 		".gz", 
+		".iso",
 		".jar",
 		".lzh",
 		".lzo",
