@@ -336,7 +336,7 @@ fr_command_lha_finalize (GObject *object)
 GType
 fr_command_lha_get_type ()
 {
-        static guint type = 0;
+        static GType type = 0;
 
         if (! type) {
                 GTypeInfo type_info = {
